@@ -3,18 +3,16 @@
 import datetime
 import json
 import logging
+import ssl
 from dataclasses import InitVar, dataclass, field
-from typing import List, Optional
-
-import httpx
+from typing import List, Optional, Union
 
 from bimmer_connected.api.authentication import MyBMWAuthentication
 from bimmer_connected.api.client import RESPONSE_STORE, MyBMWClient, MyBMWClientConfiguration
 from bimmer_connected.api.regions import Regions
 from bimmer_connected.const import (
-    ATTR_CAPABILITIES,
-    VEHICLE_CHARGING_DETAILS_URL,
-    VEHICLE_STATE_URL,
+    ATTR_ATTRIBUTES,
+    VEHICLE_PROFILE_URL,
     VEHICLES_URL,
     CarBrands,
 )
@@ -48,12 +46,18 @@ class MyBMWAccount:
     observer_position: InitVar[GPSPosition] = None
     """Optional. Required for getting a position on older cars."""
 
+    verify: InitVar[Union[ssl.SSLContext, str, bool]] = True
+    """Optional. Specify SSL context (required for Home Assistant)."""
+
     use_metric_units: InitVar[Optional[bool]] = None
     """Deprecated. All returned values are metric units (km, l)."""
 
+    hcaptcha_token: InitVar[Optional[str]] = None
+    """Optional. Required for North America region."""
+
     vehicles: List[MyBMWVehicle] = field(default_factory=list, init=False)
 
-    def __post_init__(self, password, log_responses, observer_position, use_metric_units):
+    def __post_init__(self, password, log_responses, observer_position, verify, use_metric_units, hcaptcha_token):
         """Initialize the account."""
 
         if use_metric_units is not None:
@@ -64,9 +68,10 @@ class MyBMWAccount:
 
         if self.config is None:
             self.config = MyBMWClientConfiguration(
-                MyBMWAuthentication(self.username, password, self.region),
+                MyBMWAuthentication(self.username, password, self.region, verify=verify, hcaptcha_token=hcaptcha_token),
                 log_responses=log_responses,
                 observer_position=observer_position,
+                verify=verify,
             )
 
     async def _init_vehicles(self) -> None:
@@ -76,89 +81,61 @@ class MyBMWAccount:
         fetched_at = datetime.datetime.now(datetime.timezone.utc)
 
         async with MyBMWClient(self.config) as client:
-            vehicles_responses: List[httpx.Response] = [
-                await client.get(
+            for brand in CarBrands:
+                request_headers = client.generate_default_header(brand)
+                vehicle_list_response = await client.post(
                     VEHICLES_URL,
-                    headers={
-                        **client.generate_default_header(brand),
-                    },
+                    headers=request_headers,
                 )
-                for brand in CarBrands
-            ]
 
-            for response in vehicles_responses:
-                for vehicle_base in response.json():
-                    self.add_vehicle(vehicle_base, None, None, fetched_at)
+                for vehicle in vehicle_list_response.json()["mappingInfos"]:
+                    vehicle_profile_response = await client.get(
+                        VEHICLE_PROFILE_URL, headers=dict(request_headers, **{"bmw-vin": vehicle["vin"]})
+                    )
+                    vehicle_profile = vehicle_profile_response.json()
+
+                    # Special handling for DRITTKUNDE (third party customer) aka Toyota Supra.
+                    # Requires TOYOTA in request, but returns DRITTKUNDE in response.
+                    if brand == CarBrands.TOYOTA:
+                        vehicle_profile["brand"] = CarBrands.TOYOTA.value.upper()
+
+                    vehicle_base = dict(
+                        {ATTR_ATTRIBUTES: {k: v for k, v in vehicle_profile.items() if k != "vin"}},
+                        **{"vin": vehicle_profile["vin"]},
+                    )
+
+                    await self.add_vehicle(vehicle_base, fetched_at)
 
     async def get_vehicles(self, force_init: bool = False) -> None:
         """Retrieve vehicle data from BMW servers."""
         _LOGGER.debug("Getting vehicle list")
 
-        fetched_at = datetime.datetime.now(datetime.timezone.utc)
-
         if len(self.vehicles) == 0 or force_init:
             await self._init_vehicles()
 
-        async with MyBMWClient(self.config) as client:
-            error_count = 0
-            for vehicle in self.vehicles:
-                # Get the detailed vehicle state
-                try:
-                    state_response = await client.get(
-                        VEHICLE_STATE_URL,
-                        params={
-                            "apptimezone": self.utcdiff,
-                            "appDateTime": int(fetched_at.timestamp() * 1000),
-                        },
-                        headers={
-                            **client.generate_default_header(vehicle.brand),
-                            "bmw-vin": vehicle.vin,
-                        },
-                    )
-                    vehicle_state = state_response.json()
+        error_count = 0
+        for vehicle in self.vehicles:
+            # Get the detailed vehicle state
+            try:
+                await vehicle.get_vehicle_state()
+            except (MyBMWAPIError, json.JSONDecodeError) as ex:
+                # We don't want to fail completely if one vehicle fails, but we want to know about it
+                error_count += 1
 
-                    # Get detailed charging settings if supported by vehicle
-                    charging_settings = None
-                    if vehicle_state[ATTR_CAPABILITIES].get("isChargingPlanSupported", False) or vehicle_state[
-                        ATTR_CAPABILITIES
-                    ].get("isChargingSettingsEnabled", False):
-                        charging_settings_response = await client.get(
-                            VEHICLE_CHARGING_DETAILS_URL,
-                            params={
-                                "fields": "charging-profile",
-                                "has_charging_settings_capabilities": vehicle_state[ATTR_CAPABILITIES][
-                                    "isChargingSettingsEnabled"
-                                ],
-                            },
-                            headers={
-                                **client.generate_default_header(vehicle.brand),
-                                "bmw-current-date": fetched_at.isoformat(),
-                                "bmw-vin": vehicle.vin,
-                            },
-                        )
-                        charging_settings = charging_settings_response.json()
+                # If it's a MyBMWQuotaError or MyBMWAuthError, we want to raise it
+                if isinstance(ex, (MyBMWQuotaError, MyBMWAuthError)):
+                    raise ex
 
-                    self.add_vehicle(vehicle.data, vehicle_state, charging_settings, fetched_at)
-                except (MyBMWAPIError, json.JSONDecodeError) as ex:
-                    # We don't want to fail completely if one vehicle fails, but we want to know about it
-                    error_count += 1
+                # Always log the error
+                _LOGGER.error("Unable to get details for vehicle %s - (%s) %s", vehicle.vin, type(ex).__name__, ex)
 
-                    # If it's a MyBMWQuotaError or MyBMWAuthError, we want to raise it
-                    if isinstance(ex, (MyBMWQuotaError, MyBMWAuthError)):
-                        raise ex
+                # If all vehicles fail, we want to raise an exception
+                if error_count == len(self.vehicles):
+                    raise ex
 
-                    # Always log the error
-                    _LOGGER.error("Unable to get details for vehicle %s - (%s) %s", vehicle.vin, type(ex).__name__, ex)
-
-                    # If all vehicles fail, we want to raise an exception
-                    if error_count == len(self.vehicles):
-                        raise ex
-
-    def add_vehicle(
+    async def add_vehicle(
         self,
         vehicle_base: dict,
-        vehicle_state: Optional[dict],
-        charging_settings: Optional[dict],
         fetched_at: Optional[datetime.datetime] = None,
     ) -> None:
         """Add or update a vehicle from the API responses."""
@@ -167,9 +144,9 @@ class MyBMWAccount:
 
         # If vehicle already exists, just update it's state
         if existing_vehicle:
-            existing_vehicle.update_state(vehicle_base, vehicle_state, charging_settings, fetched_at)
+            await existing_vehicle.get_vehicle_state()
         else:
-            self.vehicles.append(MyBMWVehicle(self, vehicle_base, vehicle_state, charging_settings, fetched_at))
+            self.vehicles.append(MyBMWVehicle(self, vehicle_base, fetched_at))
 
     def get_vehicle(self, vin: str) -> Optional[MyBMWVehicle]:
         """Get vehicle with given VIN.
@@ -185,12 +162,23 @@ class MyBMWAccount:
 
     def set_observer_position(self, latitude: float, longitude: float) -> None:
         """Set the position of the observer for all vehicles."""
-        self.config.observer_position = GPSPosition(latitude=latitude, longitude=longitude)
+        self.config.observer_position = GPSPosition.init_nonempty(latitude=latitude, longitude=longitude)
 
-    def set_refresh_token(self, refresh_token: str, gcid: Optional[str] = None) -> None:
-        """Overwrite the current value of the MyBMW refresh token and GCID (if available)."""
+    def set_refresh_token(
+        self,
+        refresh_token: str,
+        gcid: Optional[str] = None,
+        access_token: Optional[str] = None,
+        session_id: Optional[str] = None,
+    ) -> None:
+        """Overwrite the current value of the MyBMW tokens and GCID (if available)."""
         self.config.authentication.refresh_token = refresh_token
-        self.config.authentication.gcid = gcid
+        if gcid:
+            self.config.authentication.gcid = gcid
+        if access_token:
+            self.config.authentication.access_token = access_token
+        if session_id:
+            self.config.authentication.session_id = session_id
 
     @staticmethod
     def get_stored_responses() -> List[AnonymizedResponse]:
@@ -199,16 +187,6 @@ class MyBMWAccount:
         RESPONSE_STORE.clear()
         return responses
 
-    @property
-    def timezone(self):
-        """Returns the current tzinfo."""
-        return datetime.datetime.now().astimezone().tzinfo
-
-    @property
-    def utcdiff(self):
-        """Returns the difference to UTC in minutes."""
-        return round(self.timezone.utcoffset(datetime.datetime.now()).seconds / 60, 0)
-
     @property
     def refresh_token(self) -> Optional[str]:
         """Returns the current refresh_token."""
