@@ -5,8 +5,9 @@ import base64
 import datetime
 import logging
 import math
+import ssl
 from collections import defaultdict
-from typing import AsyncGenerator, Generator, Optional
+from typing import AsyncGenerator, Generator, Optional, Union
 from uuid import uuid4
 
 import httpx
@@ -21,6 +22,7 @@ from bimmer_connected.api.utils import (
     generate_token,
     get_capture_position,
     get_correlation_id,
+    get_x_user_agent_buildstring,
     handle_httpstatuserror,
     try_import_pillow_image,
 )
@@ -34,7 +36,7 @@ from bimmer_connected.const import (
     OAUTH_CONFIG_URL,
     X_USER_AGENT,
 )
-from bimmer_connected.models import MyBMWAPIError
+from bimmer_connected.models import MyBMWAPIError, MyBMWCaptchaMissingError
 
 EXPIRES_AT_OFFSET = datetime.timedelta(seconds=HTTPX_TIMEOUT * 2)
 
@@ -53,6 +55,8 @@ class MyBMWAuthentication(httpx.Auth):
         expires_at: Optional[datetime.datetime] = None,
         refresh_token: Optional[str] = None,
         gcid: Optional[str] = None,
+        hcaptcha_token: Optional[str] = None,
+        verify: Union[ssl.SSLContext, str, bool] = True,
     ):
         self.username: str = username
         self.password: str = password
@@ -63,6 +67,10 @@ class MyBMWAuthentication(httpx.Auth):
         self.session_id: str = str(uuid4())
         self._lock: Optional[asyncio.Lock] = None
         self.gcid: Optional[str] = gcid
+        self.hcaptcha_token: Optional[str] = hcaptcha_token
+        # Use external SSL context. Required in Home Assistant due to event loop blocking when httpx loads
+        # SSL certificates from disk. If not given, uses httpx defaults.
+        self.verify: Union[ssl.SSLContext, str, bool] = verify
 
     @property
     def login_lock(self) -> asyncio.Lock:
@@ -83,33 +91,41 @@ class MyBMWAuthentication(httpx.Auth):
         request.headers["bmw-session-id"] = self.session_id
 
         # Try getting a response
-        response: httpx.Response = (yield request)
+        response: httpx.Response = yield request
+
+        # return directly if first response was successful
+        if response.is_success:
+            return
+
         await response.aread()
 
-        # Handle "classic" 401 Unauthorized
+        # First check against 429 Too Many Requests and 403 Quota Exceeded
+        retry_count = 0
+        while (
+            response.status_code == 429 or (response.status_code == 403 and "quota" in response.text.lower())
+        ) and retry_count < 3:
+            # Quota errors can either be 429 Too Many Requests or 403 Quota Exceeded (instead of 403 Forbidden)
+            wait_time = get_retry_wait_time(response)
+            _LOGGER.debug("Sleeping %s seconds due to 429 Too Many Requests", wait_time)
+            await asyncio.sleep(wait_time)
+            response = yield request
+            await response.aread()
+            retry_count += 1
+
+        # Handle 401 Unauthorized and try getting a new token
         if response.status_code == 401:
             async with self.login_lock:
                 _LOGGER.debug("Received unauthorized response, refreshing token.")
                 await self.login()
             request.headers["authorization"] = f"Bearer {self.access_token}"
             request.headers["bmw-session-id"] = self.session_id
-            yield request
-        # Quota errors can either be 429 Too Many Requests or 403 Quota Exceeded (instead of 403 Forbidden)
-        elif response.status_code == 429 or (response.status_code == 403 and "quota" in response.text.lower()):
-            for _ in range(3):
-                if response.status_code == 429:
-                    await response.aread()
-                    wait_time = math.ceil(
-                        next(iter([int(i) for i in response.json().get("message", "") if i.isdigit()]), 2) * 1.25
-                    )
-                    _LOGGER.debug("Sleeping %s seconds due to 429 Too Many Requests", wait_time)
-                    await asyncio.sleep(wait_time)
-                    response = yield request
-            # Raise if still error after 3rd retry
-            try:
-                response.raise_for_status()
-            except httpx.HTTPStatusError as ex:
-                await handle_httpstatuserror(ex, log_handler=_LOGGER)
+            response = yield request
+
+        # Raise if request still was not successful
+        try:
+            response.raise_for_status()
+        except httpx.HTTPStatusError as ex:
+            await handle_httpstatuserror(ex, module="API", log_handler=_LOGGER)
 
     async def login(self) -> None:
         """Get a valid OAuth token."""
@@ -129,17 +145,22 @@ class MyBMWAuthentication(httpx.Auth):
             if not token_data:
                 token_data = await self._login_china()
             token_data["expires_at"] = token_data["expires_at"] - EXPIRES_AT_OFFSET
-            self.gcid = token_data["gcid"]
 
         self.access_token = token_data["access_token"]
         self.expires_at = token_data["expires_at"]
         self.refresh_token = token_data["refresh_token"]
+        self.gcid = token_data["gcid"]
 
     async def _login_row_na(self):
         """Login to Rest of World and North America."""
-        async with MyBMWLoginClient(region=self.region) as client:
+        async with MyBMWLoginClient(region=self.region, verify=self.verify) as client:
             _LOGGER.debug("Authenticating with MyBMW flow for North America & Rest of World.")
 
+            if not self.hcaptcha_token:
+                raise MyBMWCaptchaMissingError(
+                    "Missing hCaptcha token for login. See https://bimmer-connected.readthedocs.io/en/stable/captcha.html"
+                )
+
             # Get OAuth2 settings from BMW API
             r_oauth_settings = await client.get(
                 OAUTH_CONFIG_URL,
@@ -156,33 +177,42 @@ class MyBMWAuthentication(httpx.Auth):
             code_challenge = create_s256_code_challenge(code_verifier)
 
             state = generate_token(22)
+            nonce = generate_token(22)
 
             # Set up authenticate endpoint
             authenticate_url = oauth_settings["tokenEndpoint"].replace("/token", "/authenticate")
             oauth_base_values = {
                 "client_id": oauth_settings["clientId"],
                 "response_type": "code",
+                "scope": " ".join(oauth_settings["scopes"]),
                 "redirect_uri": oauth_settings["returnUrl"],
                 "state": state,
-                "nonce": "login_nonce",
-                "scope": " ".join(oauth_settings["scopes"]),
+                "nonce": nonce,
                 "code_challenge": code_challenge,
                 "code_challenge_method": "S256",
             }
 
+            authenticate_headers = {
+                "hcaptchatoken": self.hcaptcha_token,
+            }
             # Call authenticate endpoint first time (with user/pw) and get authentication
-            response = await client.post(
-                authenticate_url,
-                data=dict(
-                    oauth_base_values,
-                    **{
-                        "grant_type": "authorization_code",
-                        "username": self.username,
-                        "password": self.password,
-                    },
-                ),
-            )
-            authorization = httpx.URL(response.json()["redirect_to"]).params["authorization"]
+            try:
+                response = await client.post(
+                    authenticate_url,
+                    headers=authenticate_headers,
+                    data=dict(
+                        oauth_base_values,
+                        **{
+                            "grant_type": "authorization_code",
+                            "username": self.username,
+                            "password": self.password,
+                        },
+                    ),
+                )
+                authorization = httpx.URL(response.json()["redirect_to"]).params["authorization"]
+            finally:
+                # Always reset hCaptcha token after first login attempt
+                self.hcaptcha_token = None
 
             # With authorization, call authenticate endpoint second time to get code
             response = await client.post(
@@ -190,7 +220,10 @@ class MyBMWAuthentication(httpx.Auth):
                 params={
                     "interaction-id": uuid4(),
                     "client-version": X_USER_AGENT.format(
-                        brand="bmw", app_version=get_app_version(self.region), region=self.region.value
+                        build_string=get_x_user_agent_buildstring(),
+                        brand="bmw",
+                        app_version=get_app_version(self.region),
+                        region=self.region.value,
                     ),
                 },
                 data=dict(oauth_base_values, **{"authorization": authorization}),
@@ -218,12 +251,13 @@ class MyBMWAuthentication(httpx.Auth):
             "access_token": response_json["access_token"],
             "expires_at": expires_at,
             "refresh_token": response_json["refresh_token"],
+            "gcid": response_json["gcid"],
         }
 
     async def _refresh_token_row_na(self):
         """Login to Rest of World and North America using existing refresh_token."""
         try:
-            async with MyBMWLoginClient(region=self.region) as client:
+            async with MyBMWLoginClient(region=self.region, verify=self.verify) as client:
                 _LOGGER.debug("Authenticating with refresh token for North America & Rest of World.")
 
                 # Get OAuth2 settings from BMW API
@@ -262,10 +296,11 @@ class MyBMWAuthentication(httpx.Auth):
             "access_token": response_json["access_token"],
             "expires_at": expires_at,
             "refresh_token": response_json["refresh_token"],
+            "gcid": response_json["gcid"],
         }
 
     async def _login_china(self):
-        async with MyBMWLoginClient(region=self.region) as client:
+        async with MyBMWLoginClient(region=self.region, verify=self.verify) as client:
             _LOGGER.debug("Authenticating with MyBMW flow for China.")
 
             # While PIL.Image is only needed in `get_capture_position`, we test it here to avoid
@@ -318,7 +353,7 @@ class MyBMWAuthentication(httpx.Auth):
 
     async def _refresh_token_china(self):
         try:
-            async with MyBMWLoginClient(region=self.region) as client:
+            async with MyBMWLoginClient(region=self.region, verify=self.verify) as client:
                 _LOGGER.debug("Authenticating with refresh token for China.")
 
                 current_utc_time = datetime.datetime.now(tz=datetime.timezone.utc)
@@ -363,7 +398,12 @@ class MyBMWLoginClient(httpx.AsyncClient):
         kwargs["base_url"] = get_server_url(region)
         kwargs["headers"] = {
             "user-agent": get_user_agent(region),
-            "x-user-agent": X_USER_AGENT.format(brand="bmw", app_version=get_app_version(region), region=region.value),
+            "x-user-agent": X_USER_AGENT.format(
+                build_string=get_x_user_agent_buildstring(),
+                brand="bmw",
+                app_version=get_app_version(region),
+                region=region.value,
+            ),
         }
 
         # Register event hooks
@@ -379,7 +419,7 @@ class MyBMWLoginClient(httpx.AsyncClient):
                 try:
                     response.raise_for_status()
                 except httpx.HTTPStatusError as ex:
-                    await handle_httpstatuserror(ex, log_handler=_LOGGER)
+                    await handle_httpstatuserror(ex, module="AUTH", log_handler=_LOGGER)
 
         kwargs["event_hooks"]["response"].append(raise_for_status_event_handler)
 
@@ -394,14 +434,12 @@ class MyBMWLoginRetry(httpx.Auth):
 
     async def async_auth_flow(self, request: httpx.Request) -> AsyncGenerator[httpx.Request, httpx.Response]:
         # Try getting a response
-        response: httpx.Response = (yield request)
+        response: httpx.Response = yield request
 
         for _ in range(3):
             if response.status_code == 429:
                 await response.aread()
-                wait_time = math.ceil(
-                    next(iter([int(i) for i in response.json().get("message", "") if i.isdigit()]), 2) * 1.25
-                )
+                wait_time = get_retry_wait_time(response)
                 _LOGGER.debug("Sleeping %s seconds due to 429 Too Many Requests", wait_time)
                 await asyncio.sleep(wait_time)
                 response = yield request
@@ -411,4 +449,14 @@ class MyBMWLoginRetry(httpx.Auth):
             try:
                 response.raise_for_status()
             except httpx.HTTPStatusError as ex:
-                await handle_httpstatuserror(ex, log_handler=_LOGGER)
+                await handle_httpstatuserror(ex, module="AUTH", log_handler=_LOGGER)
+
+
+def get_retry_wait_time(response: httpx.Response) -> int:
+    """Get the wait time for the next retry from the response and multiply by 2."""
+    try:
+        response_wait_time = next(iter([int(i) for i in response.json().get("message", "") if i.isdigit()]))
+    except Exception:
+        response_wait_time = 2
+    wait_time = math.ceil(response_wait_time * 2)
+    return wait_time
