@@ -1,15 +1,21 @@
 """Generic API management."""
 
 import logging
+import ssl
 from collections import defaultdict, deque
 from dataclasses import dataclass
-from typing import Deque, Dict, Optional
+from typing import Deque, Dict, Optional, Union
 
 import httpx
 
 from bimmer_connected.api.authentication import MyBMWAuthentication
 from bimmer_connected.api.regions import get_app_version, get_server_url, get_user_agent
-from bimmer_connected.api.utils import anonymize_response, get_correlation_id, handle_httpstatuserror
+from bimmer_connected.api.utils import (
+    anonymize_response,
+    get_correlation_id,
+    get_x_user_agent_buildstring,
+    handle_httpstatuserror,
+)
 from bimmer_connected.const import HTTPX_TIMEOUT, X_USER_AGENT, CarBrands
 from bimmer_connected.models import AnonymizedResponse, GPSPosition
 
@@ -25,6 +31,7 @@ class MyBMWClientConfiguration:
     authentication: MyBMWAuthentication
     log_responses: Optional[bool] = False
     observer_position: Optional[GPSPosition] = None
+    verify: Union[ssl.SSLContext, str, bool] = True
 
     def set_log_responses(self, log_responses: bool) -> None:
         """Set if responses are logged and clear response store."""
@@ -45,6 +52,10 @@ class MyBMWClient(httpx.AsyncClient):
         # Increase timeout
         kwargs["timeout"] = httpx.Timeout(HTTPX_TIMEOUT)
 
+        # Use external SSL context stored in MyBMWClientConfiguration. Required in Home Assistant due to event loop
+        # blocking when httpx loads SSL certificates from disk. If not given, uses httpx defaults.
+        kwargs["verify"] = self.config.verify
+
         # Set default values
         kwargs["base_url"] = kwargs.get("base_url") or get_server_url(config.authentication.region)
         kwargs["headers"] = kwargs.get("headers") or self.generate_default_header(brand)
@@ -81,13 +92,15 @@ class MyBMWClient(httpx.AsyncClient):
         return {
             "accept": "application/json",
             "accept-language": "en",
+            "x-raw-locale": "en-US",
             "user-agent": get_user_agent(self.config.authentication.region),
             "x-user-agent": X_USER_AGENT.format(
+                build_string=get_x_user_agent_buildstring(),
                 brand=(brand or CarBrands.BMW).value,
                 app_version=get_app_version(self.config.authentication.region),
                 region=self.config.authentication.region.value,
             ),
             **get_correlation_id(),
-            "bmw-units-preferences": "d=KM;v=L",
+            "bmw-units-preferences": "d=KM;v=L;p=B;ec=KWH100KM;fc=L100KM;em=GKM;",
             "24-hour-format": "true",
         }
