@@ -8,9 +8,11 @@ import logging
 import sys
 import time
 from pathlib import Path
+from typing import Dict, Optional
 
 import httpx
 
+from bimmer_connected import __version__
 from bimmer_connected.account import MyBMWAccount
 from bimmer_connected.api.regions import get_region_from_name, valid_regions
 from bimmer_connected.const import DEFAULT_POI_NAME
@@ -23,16 +25,30 @@ TEXT_VIN = "Vehicle Identification Number"
 
 def main_parser() -> argparse.ArgumentParser:
     """Create the ArgumentParser with all relevant subparsers."""
-    logging.basicConfig(level=logging.DEBUG)
+    parser = argparse.ArgumentParser(
+        description=(f"Connect to MyBMW/MINI API and interact with your vehicle.\n\nVersion: {__version__}"),
+        formatter_class=argparse.RawTextHelpFormatter,
+    )
+    parser.add_argument("--debug", help="Print debug logs.", action="store_true")
+    parser.add_argument(
+        "--oauth-store",
+        help="Path to the OAuth2 storage file. Defaults to $HOME/.bimmer_connected.json.",
+        nargs="?",
+        metavar="FILE",
+        type=Path,
+        default=Path.home() / ".bimmer_connected.json",
+    )
+    parser.add_argument("--disable-oauth-store", help="Disable storing the OAuth2 tokens.", action="store_true")
 
-    parser = argparse.ArgumentParser(description="A simple executable to use and test the library.")
-    subparsers = parser.add_subparsers(dest="cmd")
+    subparsers = parser.add_subparsers(dest="cmd", description="Command", required=True)
     subparsers.required = True
 
     status_parser = subparsers.add_parser("status", description="Get the current status of the vehicle.")
     status_parser.add_argument(
         "-j", "--json", help="Output as JSON only. Removes all other output.", action="store_true"
     )
+    status_parser.add_argument("-v", "--vin", help="Output data for specified VIN only.", type=str, nargs="?")
+
     _add_default_arguments(status_parser)
     _add_position_arguments(status_parser)
 
@@ -124,23 +140,25 @@ def main_parser() -> argparse.ArgumentParser:
     return parser
 
 
-async def get_status(args) -> None:
+async def get_status(account: MyBMWAccount, args) -> None:
     """Get the vehicle status."""
     if args.json:
         for handler in logging.root.handlers[:]:
             logging.root.removeHandler(handler)
 
-    account = MyBMWAccount(args.username, args.password, get_region_from_name(args.region))
-    if args.lat and args.lng:
-        account.set_observer_position(args.lat, args.lng)
     await account.get_vehicles()
 
     if args.json:
-        print(json.dumps(account.vehicles, cls=MyBMWJSONEncoder))
+        if args.vin:
+            print(json.dumps(get_vehicle_or_return(account, args.vin), cls=MyBMWJSONEncoder))
+        else:
+            print(json.dumps(account.vehicles, cls=MyBMWJSONEncoder))
     else:
         print(f"Found {len(account.vehicles)} vehicles: {','.join([v.name for v in account.vehicles])}")
 
         for vehicle in account.vehicles:
+            if args.vin and vehicle.vin != args.vin:
+                continue
             print(f"VIN: {vehicle.vin}")
             print(f"Mileage: {vehicle.mileage.value} {vehicle.mileage.unit}")
             print("Vehicle data:")
@@ -156,46 +174,36 @@ def get_vehicle_or_return(account: MyBMWAccount, vin: str) -> MyBMWVehicle:
     return vehicle
 
 
-async def fingerprint(args) -> None:
+async def fingerprint(account: MyBMWAccount, args) -> None:
     """Save the vehicle fingerprint."""
     time_dir = Path.home() / "vehicle_fingerprint" / time.strftime("%Y-%m-%d_%H-%M-%S")
     time_dir.mkdir(parents=True)
 
-    account = MyBMWAccount(
-        args.username,
-        args.password,
-        get_region_from_name(args.region),
-        log_responses=True,
-    )
-    if args.lat and args.lng:
-        account.set_observer_position(args.lat, args.lng)
+    account.config.log_responses = True
     await account.get_vehicles()
 
     log_response_store_to_file(account.get_stored_responses(), time_dir)
     print(f"fingerprint of the vehicles written to {time_dir}")
 
 
-async def light_flash(args) -> None:
+async def light_flash(account: MyBMWAccount, args) -> None:
     """Trigger the vehicle to flash its lights."""
-    account = MyBMWAccount(args.username, args.password, get_region_from_name(args.region))
     await account.get_vehicles()
     vehicle = get_vehicle_or_return(account, args.vin)
     status = await vehicle.remote_services.trigger_remote_light_flash()
     print(status.state)
 
 
-async def horn(args) -> None:
+async def horn(account: MyBMWAccount, args) -> None:
     """Trigger the vehicle to horn."""
-    account = MyBMWAccount(args.username, args.password, get_region_from_name(args.region))
     await account.get_vehicles()
     vehicle = get_vehicle_or_return(account, args.vin)
     status = await vehicle.remote_services.trigger_remote_horn()
     print(status.state)
 
 
-async def vehicle_finder(args) -> None:
+async def vehicle_finder(account: MyBMWAccount, args) -> None:
     """Trigger the vehicle finder to locate it."""
-    account = MyBMWAccount(args.username, args.password, get_region_from_name(args.region))
     account.set_observer_position(args.lat, args.lng)
     await account.get_vehicles()
     vehicle = get_vehicle_or_return(account, args.vin)
@@ -204,11 +212,10 @@ async def vehicle_finder(args) -> None:
     print({"gps_position": vehicle.vehicle_location.location, "heading": vehicle.vehicle_location.heading})
 
 
-async def chargingsettings(args) -> None:
+async def chargingsettings(account: MyBMWAccount, args) -> None:
     """Trigger a change to charging settings."""
     if not args.target_soc and not args.ac_limit:
         raise ValueError("At least one of 'charging-target' and 'ac-limit' has to be provided.")
-    account = MyBMWAccount(args.username, args.password, get_region_from_name(args.region))
     await account.get_vehicles()
     vehicle = get_vehicle_or_return(account, args.vin)
     status = await vehicle.remote_services.trigger_charging_settings_update(
@@ -217,11 +224,10 @@ async def chargingsettings(args) -> None:
     print(status.state)
 
 
-async def chargingprofile(args) -> None:
+async def chargingprofile(account: MyBMWAccount, args) -> None:
     """Trigger a change to charging profile."""
     if not args.charging_mode and not args.precondition_climate:
         raise ValueError("At least one of 'charging-mode' and 'precondition-climate' has to be provided.")
-    account = MyBMWAccount(args.username, args.password, get_region_from_name(args.region))
     await account.get_vehicles()
     vehicle = get_vehicle_or_return(account, args.vin)
     status = await vehicle.remote_services.trigger_charging_profile_update(
@@ -230,34 +236,29 @@ async def chargingprofile(args) -> None:
     print(status.state)
 
 
-async def charge(args) -> None:
+async def charge(account: MyBMWAccount, args) -> None:
     """Trigger a vehicle to start or stop charging."""
-    account = MyBMWAccount(args.username, args.password, get_region_from_name(args.region))
     await account.get_vehicles()
     vehicle = get_vehicle_or_return(account, args.vin)
     status = await getattr(vehicle.remote_services, f"trigger_charge_{args.action.lower()}")()
     print(status.state)
 
 
-async def image(args) -> None:
+async def image(account: MyBMWAccount, args) -> None:
     """Download a rendered image of the vehicle."""
-    account = MyBMWAccount(args.username, args.password, get_region_from_name(args.region))
     await account.get_vehicles()
     vehicle = get_vehicle_or_return(account, args.vin)
 
     for viewdirection in VehicleViewDirection:
         if viewdirection == VehicleViewDirection.UNKNOWN:
             continue
-        filename = str(viewdirection.name).lower() + ".png"
-        with open(filename, "wb") as output_file:
-            image_data = await vehicle.get_vehicle_image(viewdirection)
-            output_file.write(image_data)
+        filename = (Path.cwd() / str(viewdirection.name).lower()).with_suffix(".png")
+        await asyncio.to_thread(filename.write_bytes, await vehicle.get_vehicle_image(viewdirection))
         print(f"vehicle image saved to {filename}")
 
 
-async def send_poi(args) -> None:
+async def send_poi(account: MyBMWAccount, args) -> None:
     """Send Point Of Interest to car."""
-    account = MyBMWAccount(args.username, args.password, get_region_from_name(args.region))
     await account.get_vehicles()
     vehicle = get_vehicle_or_return(account, args.vin)
 
@@ -273,9 +274,8 @@ async def send_poi(args) -> None:
     await vehicle.remote_services.trigger_send_poi(poi_data)
 
 
-async def send_poi_from_address(args) -> None:
+async def send_poi_from_address(account: MyBMWAccount, args) -> None:
     """Create Point of Interest from OSM Nominatim and send to car."""
-    account = MyBMWAccount(args.username, args.password, get_region_from_name(args.region))
     await account.get_vehicles()
     vehicle = get_vehicle_or_return(account, args.vin)
 
@@ -311,6 +311,9 @@ def _add_default_arguments(parser: argparse.ArgumentParser):
     parser.add_argument("username", help="Connected Drive username")
     parser.add_argument("password", help="Connected Drive password")
     parser.add_argument("region", choices=valid_regions(), help="Region of the Connected Drive account")
+    parser.add_argument(
+        "--captcha-token", type=str, nargs="?", help="Captcha token required for North America and Rest of World."
+    )
 
 
 def _add_position_arguments(parser: argparse.ArgumentParser):
@@ -322,13 +325,69 @@ def _add_position_arguments(parser: argparse.ArgumentParser):
     parser.set_defaults(func=get_status)
 
 
+def load_oauth_store_from_file(oauth_store: Path, account: MyBMWAccount) -> Dict:
+    """Load the OAuth details from a file if it exists."""
+    if not oauth_store.exists():
+        return {}
+    try:
+        oauth_data = json.loads(oauth_store.read_text())
+    except json.JSONDecodeError:
+        return {}
+
+    session_id_timestamp = oauth_data.pop("session_id_timestamp", None)
+    # Pop session_id every 14 days to it gets recreated
+    if (time.time() - (session_id_timestamp or 0)) > 14 * 24 * 60 * 60:
+        oauth_data.pop("session_id", None)
+        session_id_timestamp = None
+
+    account.set_refresh_token(**oauth_data)
+
+    return {**oauth_data, "session_id_timestamp": session_id_timestamp}
+
+
+def store_oauth_store_to_file(
+    oauth_store: Path, account: MyBMWAccount, session_id_timestamp: Optional[float] = None
+) -> None:
+    """Store the OAuth details to a file."""
+    oauth_store.parent.mkdir(parents=True, exist_ok=True)
+    oauth_store.write_text(
+        json.dumps(
+            {
+                "refresh_token": account.config.authentication.refresh_token,
+                "gcid": account.config.authentication.gcid,
+                "access_token": account.config.authentication.access_token,
+                "session_id": account.config.authentication.session_id,
+                "session_id_timestamp": session_id_timestamp or time.time(),
+            }
+        ),
+    )
+
+
 def main():
     """Get arguments from parser and run function in event loop."""
     parser = main_parser()
     args = parser.parse_args()
 
+    if args.debug:
+        logging.basicConfig(level=logging.DEBUG)
+        logging.getLogger("asyncio").setLevel(logging.WARNING)
+
+    account = MyBMWAccount(
+        args.username, args.password, get_region_from_name(args.region), hcaptcha_token=args.captcha_token
+    )
+
+    oauth_store_data = load_oauth_store_from_file(args.oauth_store, account)
+
     loop = asyncio.get_event_loop()
-    loop.run_until_complete(args.func(args))
+    try:
+        loop.run_until_complete(args.func(account, args))
+    except Exception as ex:  # pylint: disable=broad-except
+        sys.stderr.write(f"{type(ex).__name__}: {ex}\n")
+        sys.exit(1)
+    finally:
+        # Ensure that the OAuth2 tokens are stored even if an exception occurred
+        if not args.disable_oauth_store:
+            store_oauth_store_to_file(args.oauth_store, account, oauth_store_data.get("session_id_timestamp"))
 
 
 if __name__ == "__main__":
