@@ -1,4 +1,5 @@
 """Fixtures for BMW tests."""
+
 from collections import deque
 from typing import Deque, Generator, Optional
 
@@ -11,7 +12,9 @@ from bimmer_connected.models import AnonymizedResponse
 
 from . import (
     ALL_CHARGING_SETTINGS,
+    ALL_PROFILES,
     ALL_STATES,
+    TEST_CAPTCHA,
     TEST_PASSWORD,
     TEST_REGION,
     TEST_USERNAME,
@@ -25,6 +28,7 @@ def bmw_fixture(request: pytest.FixtureRequest) -> Generator[respx.MockRouter, N
     # Now we can start patching the API calls
     router = MyBMWMockRouter(
         vehicles_to_load=getattr(request, "param", []),
+        profiles=ALL_PROFILES,
         states=ALL_STATES,
         charging_settings=ALL_CHARGING_SETTINGS,
     )
@@ -41,8 +45,17 @@ def bmw_log_all_responses(monkeypatch: pytest.MonkeyPatch):
     monkeypatch.setattr("bimmer_connected.account.RESPONSE_STORE", temp_store)
 
 
+@pytest.fixture
+def cli_home_dir(tmp_path_factory: pytest.TempPathFactory, monkeypatch: pytest.MonkeyPatch):
+    """Create a temporary home directory for the CLI tests."""
+    tmp_path = tmp_path_factory.mktemp("cli-home-")
+    monkeypatch.setattr("pathlib.Path.home", lambda: tmp_path)
+
+    return tmp_path
+
+
 async def prepare_account_with_vehicles(region: Optional[Regions] = None):
     """Initialize account and get vehicles."""
-    account = MyBMWAccount(TEST_USERNAME, TEST_PASSWORD, region or TEST_REGION)
+    account = MyBMWAccount(TEST_USERNAME, TEST_PASSWORD, region or TEST_REGION, hcaptcha_token=TEST_CAPTCHA)
     await account.get_vehicles()
     return account
