@@ -2,6 +2,7 @@
 
 import datetime
 import logging
+import re
 from pathlib import Path
 from unittest import mock
 
@@ -13,37 +14,64 @@ from bimmer_connected.account import MyBMWAccount
 from bimmer_connected.api.authentication import MyBMWAuthentication, MyBMWLoginRetry
 from bimmer_connected.api.client import MyBMWClient
 from bimmer_connected.api.regions import get_region_from_name
-from bimmer_connected.const import ATTR_CAPABILITIES
-from bimmer_connected.models import GPSPosition, MyBMWAPIError, MyBMWAuthError, MyBMWQuotaError
+from bimmer_connected.const import ATTR_CAPABILITIES, VEHICLES_URL, CarBrands, Regions
+from bimmer_connected.models import (
+    GPSPosition,
+    MyBMWAPIError,
+    MyBMWAuthError,
+    MyBMWCaptchaMissingError,
+    MyBMWQuotaError,
+)
 
 from . import (
     RESPONSE_DIR,
+    TEST_CAPTCHA,
     TEST_PASSWORD,
     TEST_REGION,
     TEST_REGION_STRING,
     TEST_USERNAME,
     VIN_G26,
     VIN_I20,
-    get_fingerprint_charging_settings_count,
-    get_fingerprint_state_count,
+    get_fingerprint_count,
     load_response,
 )
+from .common import MyBMWMockRouter
 from .conftest import prepare_account_with_vehicles
 
 
 @pytest.mark.asyncio
-async def test_login_row_na(bmw_fixture: respx.Router):
+async def test_login_row(bmw_fixture: respx.Router):
     """Test the login flow."""
-    account = MyBMWAccount(TEST_USERNAME, TEST_PASSWORD, get_region_from_name(TEST_REGION_STRING))
+    account = MyBMWAccount(
+        TEST_USERNAME, TEST_PASSWORD, get_region_from_name(TEST_REGION_STRING), hcaptcha_token=TEST_CAPTCHA
+    )
     await account.get_vehicles()
     assert account is not None
 
 
+@pytest.mark.asyncio
+async def test_login_na(bmw_fixture: respx.Router):
+    """Test the login flow for North America."""
+    account = MyBMWAccount(TEST_USERNAME, TEST_PASSWORD, Regions.NORTH_AMERICA, hcaptcha_token=TEST_CAPTCHA)
+    await account.get_vehicles()
+    assert account is not None
+
+
+@pytest.mark.asyncio
+async def test_login_na_without_hcaptcha(bmw_fixture: respx.Router):
+    """Test the login flow."""
+    with pytest.raises(MyBMWCaptchaMissingError):
+        account = MyBMWAccount(TEST_USERNAME, TEST_PASSWORD, Regions.NORTH_AMERICA)
+        await account.get_vehicles()
+
+
 @pytest.mark.asyncio
 async def test_login_refresh_token_row_na_expired(bmw_fixture: respx.Router):
     """Test the login flow using refresh_token."""
     with mock.patch("bimmer_connected.api.authentication.EXPIRES_AT_OFFSET", datetime.timedelta(seconds=30000)):
-        account = MyBMWAccount(TEST_USERNAME, TEST_PASSWORD, get_region_from_name(TEST_REGION_STRING))
+        account = MyBMWAccount(
+            TEST_USERNAME, TEST_PASSWORD, get_region_from_name(TEST_REGION_STRING), hcaptcha_token=TEST_CAPTCHA
+        )
         await account.get_vehicles()
 
         with mock.patch(
@@ -62,7 +90,9 @@ async def test_login_refresh_token_row_na_expired(bmw_fixture: respx.Router):
 async def test_login_refresh_token_row_na_401(bmw_fixture: respx.Router):
     """Test the login flow using refresh_token."""
 
-    account = MyBMWAccount(TEST_USERNAME, TEST_PASSWORD, get_region_from_name(TEST_REGION_STRING))
+    account = MyBMWAccount(
+        TEST_USERNAME, TEST_PASSWORD, get_region_from_name(TEST_REGION_STRING), hcaptcha_token=TEST_CAPTCHA
+    )
     await account.get_vehicles()
 
     with mock.patch(
@@ -89,7 +119,9 @@ async def test_login_refresh_token_row_na_invalid(caplog, bmw_fixture: respx.Rou
         ]
     )
 
-    account = MyBMWAccount(TEST_USERNAME, TEST_PASSWORD, get_region_from_name(TEST_REGION_STRING))
+    account = MyBMWAccount(
+        TEST_USERNAME, TEST_PASSWORD, get_region_from_name(TEST_REGION_STRING), hcaptcha_token=TEST_CAPTCHA
+    )
     account.set_refresh_token("INVALID")
 
     caplog.set_level(logging.DEBUG)
@@ -177,11 +209,11 @@ async def test_vehicles(bmw_fixture: respx.Router):
     await account.get_vehicles()
 
     assert account.config.authentication.access_token is not None
-    assert get_fingerprint_state_count() == len(account.vehicles)
+    assert get_fingerprint_count("profiles") == len(account.vehicles)
 
     vehicle = account.get_vehicle(VIN_G26)
     assert vehicle is not None
-    assert VIN_G26 == vehicle.vin
+    assert vehicle.vin == VIN_G26
 
     assert account.get_vehicle("invalid_vin") is None
 
@@ -189,7 +221,7 @@ async def test_vehicles(bmw_fixture: respx.Router):
 @pytest.mark.asyncio
 async def test_vehicle_init(bmw_fixture: respx.Router):
     """Test vehicle initialization."""
-    account = MyBMWAccount(TEST_USERNAME, TEST_PASSWORD, TEST_REGION)
+    account = MyBMWAccount(TEST_USERNAME, TEST_PASSWORD, TEST_REGION, hcaptcha_token=TEST_CAPTCHA)
     with mock.patch(
         "bimmer_connected.account.MyBMWAccount._init_vehicles",
         wraps=account._init_vehicles,
@@ -198,15 +230,15 @@ async def test_vehicle_init(bmw_fixture: respx.Router):
 
         # First call on init
         await account.get_vehicles()
-        assert len(account.vehicles) == get_fingerprint_state_count()
+        assert len(account.vehicles) == get_fingerprint_count("profiles")
 
         # No call to _init_vehicles()
         await account.get_vehicles()
-        assert len(account.vehicles) == get_fingerprint_state_count()
+        assert len(account.vehicles) == get_fingerprint_count("profiles")
 
         # Second, forced call _init_vehicles()
         await account.get_vehicles(force_init=True)
-        assert len(account.vehicles) == get_fingerprint_state_count()
+        assert len(account.vehicles) == get_fingerprint_count("profiles")
 
         assert mock_listener.call_count == 2
 
@@ -218,7 +250,7 @@ async def test_invalid_password(bmw_fixture: respx.Router):
         401, json=load_response(RESPONSE_DIR / "auth" / "auth_error_wrong_password.json")
     )
     with pytest.raises(MyBMWAuthError):
-        account = MyBMWAccount(TEST_USERNAME, TEST_PASSWORD, TEST_REGION)
+        account = MyBMWAccount(TEST_USERNAME, TEST_PASSWORD, TEST_REGION, hcaptcha_token=TEST_CAPTCHA)
         await account.get_vehicles()
 
 
@@ -240,7 +272,7 @@ async def test_server_error(bmw_fixture: respx.Router):
         500, text=load_response(RESPONSE_DIR / "auth" / "auth_error_internal_error.txt")
     )
     with pytest.raises(MyBMWAPIError):
-        account = MyBMWAccount(TEST_USERNAME, TEST_PASSWORD, TEST_REGION)
+        account = MyBMWAccount(TEST_USERNAME, TEST_PASSWORD, TEST_REGION, hcaptcha_token=TEST_CAPTCHA)
         await account.get_vehicles()
 
 
@@ -259,11 +291,16 @@ async def test_vehicle_search_case(bmw_fixture: respx.Router):
 async def test_get_fingerprints(monkeypatch: pytest.MonkeyPatch, bmw_fixture: respx.Router, bmw_log_all_responses):
     """Test getting fingerprints."""
 
-    # Prepare Number of good responses (vehicle states, charging settings per vehicle)
+    # Prepare Number of good responses (vehicle profiles + vehicle states, charging settings per vehicle)
     # and 2x vehicle list
-    json_count = get_fingerprint_state_count() + get_fingerprint_charging_settings_count() + 2
+    json_count = (
+        get_fingerprint_count("vehicles")
+        + get_fingerprint_count("profiles")
+        + get_fingerprint_count("states")
+        + get_fingerprint_count("charging_settings")
+    )
 
-    account = MyBMWAccount(TEST_USERNAME, TEST_PASSWORD, TEST_REGION, log_responses=True)
+    account = MyBMWAccount(TEST_USERNAME, TEST_PASSWORD, TEST_REGION, hcaptcha_token=TEST_CAPTCHA, log_responses=True)
     await account.get_vehicles()
 
     # This should have been successful
@@ -283,7 +320,7 @@ async def test_get_fingerprints(monkeypatch: pytest.MonkeyPatch, bmw_fixture: re
     )
     bmw_fixture.routes.add(state_route, "state")
 
-    account = MyBMWAccount(TEST_USERNAME, TEST_PASSWORD, TEST_REGION, log_responses=True)
+    account = MyBMWAccount(TEST_USERNAME, TEST_PASSWORD, TEST_REGION, hcaptcha_token=TEST_CAPTCHA, log_responses=True)
     await account.get_vehicles()
 
     filenames = [Path(f.filename) for f in account.get_stored_responses()]
@@ -321,14 +358,14 @@ async def test_set_observer_invalid_values(bmw_fixture: respx.Router):
     """Test set_observer_position with invalid arguments."""
     account = await prepare_account_with_vehicles()
 
-    with pytest.raises(TypeError):
-        account.set_observer_position(None, 2.0)
+    with pytest.raises(ValueError, match="requires both 'latitude' and 'longitude' set"):
+        account.set_observer_position(1, None)
 
-    with pytest.raises(TypeError):
-        account.set_observer_position(1.0, None)
+    with pytest.raises(ValueError, match="requires both 'latitude' and 'longitude' set"):
+        account.set_observer_position(None, None)
 
-    with pytest.raises(TypeError):
-        account.set_observer_position(1.0, "16.0")
+    account.set_observer_position(1, 2)
+    assert account.config.observer_position == GPSPosition(1.0, 2.0)
 
 
 @pytest.mark.asyncio
@@ -336,38 +373,44 @@ async def test_set_use_metric_units(caplog):
     """Test (deprecated) use_metrics_units flag."""
 
     # Default
-    account = MyBMWAccount(TEST_USERNAME, TEST_PASSWORD, TEST_REGION)
+    account = MyBMWAccount(TEST_USERNAME, TEST_PASSWORD, TEST_REGION, hcaptcha_token=TEST_CAPTCHA)
     assert len(caplog.records) == 0
     metric_client = MyBMWClient(account.config)
-    assert metric_client.generate_default_header()["bmw-units-preferences"] == "d=KM;v=L"
+    assert (
+        metric_client.generate_default_header()["bmw-units-preferences"] == "d=KM;v=L;p=B;ec=KWH100KM;fc=L100KM;em=GKM;"
+    )
 
     # Set to true
     caplog.clear()
     account = MyBMWAccount(TEST_USERNAME, TEST_PASSWORD, TEST_REGION, use_metric_units=True)
     assert len(caplog.records) == 1
     metric_client = MyBMWClient(account.config)
-    assert metric_client.generate_default_header()["bmw-units-preferences"] == "d=KM;v=L"
+    assert (
+        metric_client.generate_default_header()["bmw-units-preferences"] == "d=KM;v=L;p=B;ec=KWH100KM;fc=L100KM;em=GKM;"
+    )
 
     # Set to false
     caplog.clear()
     account = MyBMWAccount(TEST_USERNAME, TEST_PASSWORD, TEST_REGION, use_metric_units=True)
     assert len(caplog.records) == 1
     metric_client = MyBMWClient(account.config)
-    assert metric_client.generate_default_header()["bmw-units-preferences"] == "d=KM;v=L"
+    assert (
+        metric_client.generate_default_header()["bmw-units-preferences"] == "d=KM;v=L;p=B;ec=KWH100KM;fc=L100KM;em=GKM;"
+    )
 
 
 @pytest.mark.asyncio
 async def test_refresh_token_getset(bmw_fixture: respx.Router):
     """Test getting/setting the refresh_token and gcid."""
-    account = MyBMWAccount(TEST_USERNAME, TEST_PASSWORD, TEST_REGION)
+    account = MyBMWAccount(TEST_USERNAME, TEST_PASSWORD, TEST_REGION, hcaptcha_token=TEST_CAPTCHA)
     assert account.refresh_token is None
     await account.get_vehicles()
     assert account.refresh_token == "another_token_string"
-    assert account.gcid is None
+    assert account.gcid == "DUMMY"
 
     account.set_refresh_token("new_refresh_token")
     assert account.refresh_token == "new_refresh_token"
-    assert account.gcid is None
+    assert account.gcid == "DUMMY"
 
     account = MyBMWAccount(TEST_USERNAME, TEST_PASSWORD, get_region_from_name("china"))
     account.set_refresh_token("new_refresh_token", "dummy_gcid")
@@ -379,9 +422,9 @@ async def test_refresh_token_getset(bmw_fixture: respx.Router):
 
 
 @pytest.mark.asyncio
-async def test_429_retry_ok_login(caplog, bmw_fixture: respx.Router):
+async def test_429_retry_ok_oauth_config(caplog, bmw_fixture: respx.Router):
     """Test the login flow using refresh_token."""
-    account = MyBMWAccount(TEST_USERNAME, TEST_PASSWORD, TEST_REGION)
+    account = MyBMWAccount(TEST_USERNAME, TEST_PASSWORD, TEST_REGION, hcaptcha_token=TEST_CAPTCHA)
 
     json_429 = {"statusCode": 429, "message": "Rate limit is exceeded. Try again in 2 seconds."}
 
@@ -406,18 +449,66 @@ async def test_429_retry_ok_login(caplog, bmw_fixture: respx.Router):
 
 
 @pytest.mark.asyncio
-async def test_429_retry_raise_login(caplog, bmw_fixture: respx.Router):
+async def test_429_retry_raise_oauth_config(caplog, bmw_fixture: respx.Router):
     """Test the login flow using refresh_token."""
-    account = MyBMWAccount(TEST_USERNAME, TEST_PASSWORD, TEST_REGION)
+    account = MyBMWAccount(TEST_USERNAME, TEST_PASSWORD, TEST_REGION, hcaptcha_token=TEST_CAPTCHA)
 
     json_429 = {"statusCode": 429, "message": "Rate limit is exceeded. Try again in 2 seconds."}
 
     bmw_fixture.get("/eadrax-ucs/v1/presentation/oauth/config").mock(return_value=httpx.Response(429, json=json_429))
     caplog.set_level(logging.DEBUG)
 
+    with mock.patch("asyncio.sleep", new_callable=mock.AsyncMock), pytest.raises(MyBMWAPIError):
+        await account.get_vehicles()
+
+    log_429 = [
+        r
+        for r in caplog.records
+        if r.module == "authentication" and "seconds due to 429 Too Many Requests" in r.message
+    ]
+    assert len(log_429) == 3
+
+
+@pytest.mark.asyncio
+async def test_429_retry_ok_authenticate(caplog, bmw_fixture: respx.Router):
+    """Test the login flow using refresh_token."""
+    account = MyBMWAccount(TEST_USERNAME, TEST_PASSWORD, TEST_REGION, hcaptcha_token=TEST_CAPTCHA)
+
+    json_429 = {"statusCode": 429, "message": "Rate limit is exceeded. Try again in 2 seconds."}
+
+    bmw_fixture.post("/gcdm/oauth/authenticate").mock(
+        side_effect=[
+            httpx.Response(429, json=json_429),
+            httpx.Response(429, json=json_429),
+            MyBMWMockRouter.authenticate_sideeffect,  # type: ignore[list-item]
+            MyBMWMockRouter.authenticate_sideeffect,  # type: ignore[list-item]
+        ]
+    )
+    caplog.set_level(logging.DEBUG)
+
     with mock.patch("asyncio.sleep", new_callable=mock.AsyncMock):
-        with pytest.raises(MyBMWAPIError):
-            await account.get_vehicles()
+        await account.get_vehicles()
+
+    log_429 = [
+        r
+        for r in caplog.records
+        if r.module == "authentication" and "seconds due to 429 Too Many Requests" in r.message
+    ]
+    assert len(log_429) == 2
+
+
+@pytest.mark.asyncio
+async def test_429_retry_raise_authenticate(caplog, bmw_fixture: respx.Router):
+    """Test the login flow using refresh_token."""
+    account = MyBMWAccount(TEST_USERNAME, TEST_PASSWORD, TEST_REGION, hcaptcha_token=TEST_CAPTCHA)
+
+    json_429 = {"statusCode": 429, "message": "Rate limit is exceeded. Try again in 2 seconds."}
+
+    bmw_fixture.post("/gcdm/oauth/authenticate").mock(return_value=httpx.Response(429, json=json_429))
+    caplog.set_level(logging.DEBUG)
+
+    with mock.patch("asyncio.sleep", new_callable=mock.AsyncMock), pytest.raises(MyBMWAuthError):
+        await account.get_vehicles()
 
     log_429 = [
         r
@@ -430,15 +521,18 @@ async def test_429_retry_raise_login(caplog, bmw_fixture: respx.Router):
 @pytest.mark.asyncio
 async def test_429_retry_ok_vehicles(caplog, bmw_fixture: respx.Router):
     """Test waiting on 429 for vehicles."""
-    account = MyBMWAccount(TEST_USERNAME, TEST_PASSWORD, TEST_REGION)
+    account = MyBMWAccount(TEST_USERNAME, TEST_PASSWORD, TEST_REGION, hcaptcha_token=TEST_CAPTCHA)
 
     json_429 = {"statusCode": 429, "message": "Rate limit is exceeded. Try again in 2 seconds."}
 
-    bmw_fixture.get("/eadrax-vcs/v4/vehicles").mock(
+    bmw_fixture.post(VEHICLES_URL).mock(
         side_effect=[
             httpx.Response(429, json=json_429),
             httpx.Response(429, json=json_429),
-            *[httpx.Response(200, json=[])] * 2,
+            *[
+                httpx.Response(200, json=load_response(RESPONSE_DIR / f"{brand.value}-eadrax-vcs_v5_vehicle-list.json"))
+                for brand in CarBrands
+            ],
         ]
     )
     caplog.set_level(logging.DEBUG)
@@ -457,16 +551,15 @@ async def test_429_retry_ok_vehicles(caplog, bmw_fixture: respx.Router):
 @pytest.mark.asyncio
 async def test_429_retry_raise_vehicles(caplog, bmw_fixture: respx.Router):
     """Test waiting on 429 for vehicles and fail if it happens too often."""
-    account = MyBMWAccount(TEST_USERNAME, TEST_PASSWORD, TEST_REGION)
+    account = MyBMWAccount(TEST_USERNAME, TEST_PASSWORD, TEST_REGION, hcaptcha_token=TEST_CAPTCHA)
 
     json_429 = {"statusCode": 429, "message": "Rate limit is exceeded. Try again in 2 seconds."}
 
-    bmw_fixture.get("/eadrax-vcs/v4/vehicles").mock(return_value=httpx.Response(429, json=json_429))
+    bmw_fixture.post(VEHICLES_URL).mock(return_value=httpx.Response(429, json=json_429))
     caplog.set_level(logging.DEBUG)
 
-    with mock.patch("asyncio.sleep", new_callable=mock.AsyncMock):
-        with pytest.raises(MyBMWQuotaError):
-            await account.get_vehicles()
+    with mock.patch("asyncio.sleep", new_callable=mock.AsyncMock), pytest.raises(MyBMWQuotaError):
+        await account.get_vehicles()
 
     log_429 = [
         r
@@ -476,10 +569,117 @@ async def test_429_retry_raise_vehicles(caplog, bmw_fixture: respx.Router):
     assert len(log_429) == 3
 
 
+@pytest.mark.asyncio
+async def test_429_retry_with_login_ok_vehicles(bmw_fixture: respx.Router):
+    """Test the login flow but experiencing a 429 first."""
+    account = MyBMWAccount(TEST_USERNAME, TEST_PASSWORD, TEST_REGION, hcaptcha_token=TEST_CAPTCHA)
+
+    json_429 = {"statusCode": 429, "message": "Rate limit is exceeded. Try again in 2 seconds."}
+
+    bmw_fixture.post(VEHICLES_URL).mock(
+        side_effect=[
+            httpx.Response(429, json=json_429),
+            httpx.Response(429, json=json_429),
+            *[
+                httpx.Response(200, json=load_response(RESPONSE_DIR / f"{brand.value}-eadrax-vcs_v5_vehicle-list.json"))
+                for brand in CarBrands
+            ],
+        ]
+    )
+
+    with mock.patch("asyncio.sleep", new_callable=mock.AsyncMock):
+        await account.get_vehicles()
+
+
+@pytest.mark.asyncio
+async def test_429_retry_with_login_raise_vehicles(bmw_fixture: respx.Router):
+    """Test the error handling, experiencing a 429, 401 and another two 429."""
+    account = MyBMWAccount(TEST_USERNAME, TEST_PASSWORD, TEST_REGION, hcaptcha_token=TEST_CAPTCHA)
+
+    json_429 = {"statusCode": 429, "message": "Rate limit is exceeded. Try again in 2 seconds."}
+
+    bmw_fixture.post(VEHICLES_URL).mock(
+        side_effect=[
+            httpx.Response(429, json=json_429),
+            httpx.Response(401),
+            httpx.Response(429, json=json_429),
+            httpx.Response(429, json=json_429),
+            httpx.Response(429, json=json_429),
+            httpx.Response(429, json=json_429),
+            httpx.Response(429, json=json_429),
+        ]
+    )
+
+    with mock.patch("asyncio.sleep", new_callable=mock.AsyncMock), pytest.raises(MyBMWQuotaError):
+        await account.get_vehicles()
+
+
+@pytest.mark.asyncio
+async def test_multiple_401(bmw_fixture: respx.Router):
+    """Test the error handling, when multiple 401 are received in sequence."""
+    account = MyBMWAccount(TEST_USERNAME, TEST_PASSWORD, TEST_REGION, hcaptcha_token=TEST_CAPTCHA)
+
+    bmw_fixture.post(VEHICLES_URL).mock(
+        side_effect=[
+            httpx.Response(401),
+            httpx.Response(401),
+        ]
+    )
+
+    with mock.patch("asyncio.sleep", new_callable=mock.AsyncMock), pytest.raises(MyBMWAuthError):
+        await account.get_vehicles()
+
+
+@pytest.mark.asyncio
+async def test_401_after_429_ok(bmw_fixture: respx.Router):
+    """Test the error handling, when a 401 is received after exactly 3 429."""
+    account = MyBMWAccount(TEST_USERNAME, TEST_PASSWORD, TEST_REGION, hcaptcha_token=TEST_CAPTCHA)
+    await account.get_vehicles()
+
+    json_429 = {"statusCode": 429, "message": "Rate limit is exceeded. Try again in 2 seconds."}
+
+    # Recover after 3 429 and 1 401
+    bmw_fixture.post(VEHICLES_URL).mock(
+        side_effect=[
+            httpx.Response(429, json=json_429),
+            httpx.Response(429, json=json_429),
+            httpx.Response(429, json=json_429),
+            httpx.Response(401),
+            # Just simulate OK responses from now on
+            *[httpx.Response(200, json=load_response(RESPONSE_DIR / "bmw-eadrax-vcs_v5_vehicle-list.json"))] * 100,
+        ]
+    )
+    with mock.patch("asyncio.sleep", new_callable=mock.AsyncMock):
+        await account.get_vehicles()
+    assert len(account.vehicles) == get_fingerprint_count("profiles")
+
+
+@pytest.mark.asyncio
+async def test_401_after_429_fail(bmw_fixture: respx.Router):
+    """Test the error handling, when a 401 is received after exactly 3 429."""
+    account = MyBMWAccount(TEST_USERNAME, TEST_PASSWORD, TEST_REGION, hcaptcha_token=TEST_CAPTCHA)
+
+    json_429 = {"statusCode": 429, "message": "Rate limit is exceeded. Try again in 2 seconds."}
+
+    # Fail after 3 429 and 1 401 with another 429
+    bmw_fixture.post(VEHICLES_URL).mock(
+        side_effect=[
+            httpx.Response(429, json=json_429),
+            httpx.Response(429, json=json_429),
+            httpx.Response(429, json=json_429),
+            httpx.Response(401),
+            httpx.Response(429, json=json_429),
+        ]
+    )
+
+    with mock.patch("asyncio.sleep", new_callable=mock.AsyncMock), pytest.raises(MyBMWQuotaError):
+        await account.get_vehicles()
+
+
 @pytest.mark.asyncio
 async def test_403_quota_exceeded_vehicles_usa(caplog, bmw_fixture: respx.Router):
     """Test 403 quota issues for vehicle state and fail if it happens too often."""
-    account = MyBMWAccount(TEST_USERNAME, TEST_PASSWORD, TEST_REGION)
+    account = MyBMWAccount(TEST_USERNAME, TEST_PASSWORD, TEST_REGION, hcaptcha_token=TEST_CAPTCHA)
     # get vehicles once
     await account.get_vehicles()
 
@@ -491,9 +691,8 @@ async def test_403_quota_exceeded_vehicles_usa(caplog, bmw_fixture: respx.Router
     )
     caplog.set_level(logging.DEBUG)
 
-    with mock.patch("asyncio.sleep", new_callable=mock.AsyncMock):
-        with pytest.raises(MyBMWQuotaError):
-            await account.get_vehicles()
+    with mock.patch("asyncio.sleep", new_callable=mock.AsyncMock), pytest.raises(MyBMWQuotaError):
+        await account.get_vehicles()
 
     log_quota = [r for r in caplog.records if "quota" in r.message]
     assert len(log_quota) == 1
@@ -502,7 +701,7 @@ async def test_403_quota_exceeded_vehicles_usa(caplog, bmw_fixture: respx.Router
 @pytest.mark.asyncio
 async def test_incomplete_vehicle_details(caplog, bmw_fixture: respx.Router):
     """Test incorrect responses for vehicle details."""
-    account = MyBMWAccount(TEST_USERNAME, TEST_PASSWORD, TEST_REGION)
+    account = MyBMWAccount(TEST_USERNAME, TEST_PASSWORD, TEST_REGION, hcaptcha_token=TEST_CAPTCHA)
     # get vehicles once
     await account.get_vehicles()
 
@@ -528,7 +727,7 @@ async def test_incomplete_vehicle_details(caplog, bmw_fixture: respx.Router):
 @pytest.mark.asyncio
 async def test_no_vehicle_details(caplog, bmw_fixture: respx.Router):
     """Test raising an exception if no responses for vehicle details are received."""
-    account = MyBMWAccount(TEST_USERNAME, TEST_PASSWORD, TEST_REGION)
+    account = MyBMWAccount(TEST_USERNAME, TEST_PASSWORD, TEST_REGION, hcaptcha_token=TEST_CAPTCHA)
     await account.get_vehicles()
 
     bmw_fixture.get("/eadrax-vcs/v4/vehicles/state").mock(
@@ -541,20 +740,20 @@ async def test_no_vehicle_details(caplog, bmw_fixture: respx.Router):
         await account.get_vehicles()
 
     log_error = [r for r in caplog.records if "Unable to get details" in r.message]
-    assert len(log_error) == get_fingerprint_state_count()
+    assert len(log_error) == get_fingerprint_count("profiles")
 
 
 @pytest.mark.asyncio
 async def test_client_async_only(bmw_fixture: respx.Router):
     """Test that the Authentication providers only work async."""
 
-    with httpx.Client(auth=MyBMWAuthentication(TEST_USERNAME, TEST_PASSWORD, TEST_REGION)) as client:
-        with pytest.raises(RuntimeError):
-            client.get("/eadrax-ucs/v1/presentation/oauth/config")
+    with httpx.Client(
+        auth=MyBMWAuthentication(TEST_USERNAME, TEST_PASSWORD, TEST_REGION, hcaptcha_token=TEST_CAPTCHA)
+    ) as client, pytest.raises(RuntimeError):
+        client.get("/eadrax-ucs/v1/presentation/oauth/config")
 
-    with httpx.Client(auth=MyBMWLoginRetry()) as client:
-        with pytest.raises(RuntimeError):
-            client.get("/eadrax-ucs/v1/presentation/oauth/config")
+    with httpx.Client(auth=MyBMWLoginRetry()) as client, pytest.raises(RuntimeError):
+        client.get("/eadrax-ucs/v1/presentation/oauth/config")
 
 
 @pytest.mark.asyncio
@@ -574,12 +773,24 @@ async def test_pillow_unavailable(monkeypatch: pytest.MonkeyPatch, bmw_fixture:
     assert len(account.vehicles) == 0
 
     # But rest_of_world and north_america should work
-    account = MyBMWAccount(TEST_USERNAME, TEST_PASSWORD, get_region_from_name("rest_of_world"))
+    account = MyBMWAccount(
+        TEST_USERNAME, TEST_PASSWORD, get_region_from_name("rest_of_world"), hcaptcha_token=TEST_CAPTCHA
+    )
     await account.get_vehicles()
     assert account is not None
     assert len(account.vehicles) > 0
 
-    account = MyBMWAccount(TEST_USERNAME, TEST_PASSWORD, get_region_from_name("north_america"))
-    await account.get_vehicles()
-    assert account is not None
-    assert len(account.vehicles) > 0
+
+def test_x_user_agent():
+    """Test that X-User-Agent contains 'android' and correctly formatted build-string (PREFIX.NUMERIC.BUILD.PATCH)."""
+
+    # Default
+    account = MyBMWAccount(TEST_USERNAME, TEST_PASSWORD, TEST_REGION, hcaptcha_token=TEST_CAPTCHA)
+    metric_client = MyBMWClient(account.config)
+    header_value = metric_client.generate_default_header()["x-user-agent"]
+
+    assert "android" in header_value, f"'android' not found in X-User-Agent: {header_value}"
+
+    pattern = r"\([A-Z0-9]{4}\.[0-9]{6}\.[A-F0-9]{3}\)"
+    match = re.search(pattern, header_value)
+    assert match, f"Buildstring format incorrect in X-User-Agent: {header_value}"
