@@ -1,13 +1,15 @@
 """Test for VehicleState."""
 
 import datetime
+import os
+import time
 
 import pytest
 import respx
 import time_machine
 
 from bimmer_connected.api.regions import get_region_from_name
-from bimmer_connected.vehicle.climate import Climate, ClimateActivityState
+from bimmer_connected.vehicle.climate import ClimateActivityState
 from bimmer_connected.vehicle.doors_windows import LidState, LockState
 from bimmer_connected.vehicle.fuel_and_battery import ChargingState, FuelAndBattery
 from bimmer_connected.vehicle.location import VehicleLocation
@@ -23,6 +25,7 @@ from . import (
     VIN_I01_NOREX,
     VIN_I01_REX,
     VIN_I20,
+    VIN_J29,
     get_deprecation_warning_count,
 )
 from .conftest import prepare_account_with_vehicles
@@ -39,8 +42,8 @@ async def test_generic(caplog, bmw_fixture: respx.Router):
     expected = datetime.datetime(year=2023, month=1, day=4, hour=14, minute=57, second=6, tzinfo=UTC)
     assert expected == status.timestamp
 
-    assert 1121 == status.mileage[0]
-    assert "km" == status.mileage[1]
+    assert status.mileage[0] == 1121
+    assert status.mileage[1] == "km"
 
     assert len(get_deprecation_warning_count(caplog)) == 0
 
@@ -65,27 +68,21 @@ async def test_generic_error_handling(caplog, bmw_fixture: respx.Router):
 
     caplog.clear()
 
-    vehicle = account.get_vehicle(VIN_I20)
-    state_wo_climate_activity = vehicle.data["state"].copy()
-    state_wo_climate_activity["climateControlState"].pop("activity", None)
-    vehicle.climate = Climate(account_timezone=account.timezone)
-
-    assert vehicle.climate.activity == ClimateActivityState.UNKNOWN
-    vehicle.update_state(vehicle.data, state_wo_climate_activity)
-    assert (
-        any("climate" in r.message and "KeyError" in r.message and "'activity'" in r.message for r in caplog.records)
-        is True
-    )
-    assert vehicle.climate.activity == ClimateActivityState.UNKNOWN
-
 
 @pytest.mark.asyncio
-async def test_range_combustion_no_info(caplog, bmw_fixture: respx.Router):
-    """Test if the parsing of mileage and range is working."""
-    vehicle = (await prepare_account_with_vehicles()).get_vehicle(VIN_F31)
+@pytest.mark.parametrize(
+    ("vin"),
+    [
+        (VIN_F31),
+        (VIN_J29),
+    ],
+)
+async def test_range_combustion_no_info(caplog, bmw_fixture: respx.Router, vin: str):
+    """Test if the parsing of very old vehicles."""
+    vehicle = (await prepare_account_with_vehicles()).get_vehicle(vin)
     status = vehicle.fuel_and_battery
 
-    assert (14, "L") == status.remaining_fuel
+    assert status.remaining_fuel == (None, None)
     assert status.remaining_range_fuel == (None, None)
     assert status.remaining_fuel_percent is None
 
@@ -94,27 +91,27 @@ async def test_range_combustion_no_info(caplog, bmw_fixture: respx.Router):
 
     assert status.remaining_range_total == (None, None)
 
-    status_from_vehicle_data = FuelAndBattery.from_vehicle_data(vehicle.data)
-    status_from_vehicle_data.account_timezone = status.account_timezone
-    assert status_from_vehicle_data == status
-    assert FuelAndBattery.from_vehicle_data({}) is None
-
     assert len(get_deprecation_warning_count(caplog)) == 0
 
 
 @pytest.mark.asyncio
 async def test_range_combustion(caplog, bmw_fixture: respx.Router):
     """Test if the parsing of mileage and range is working."""
-    status = (await prepare_account_with_vehicles()).get_vehicle(VIN_G20).fuel_and_battery
+    vehicle = (await prepare_account_with_vehicles()).get_vehicle(VIN_G20)
+    status = vehicle.fuel_and_battery
 
-    assert (40, "L") == status.remaining_fuel
-    assert (629, "km") == status.remaining_range_fuel
+    assert status.remaining_fuel == (40, "L")
+    assert status.remaining_range_fuel == (629, "km")
     assert status.remaining_fuel_percent == 80
 
     assert status.remaining_battery_percent is None
     assert status.remaining_range_electric == (None, None)
 
-    assert (629, "km") == status.remaining_range_total
+    assert status.remaining_range_total == (629, "km")
+
+    status_from_vehicle_data = FuelAndBattery.from_vehicle_data(vehicle.data)
+    assert status_from_vehicle_data == status
+    assert FuelAndBattery.from_vehicle_data({}) is None
 
     assert len(get_deprecation_warning_count(caplog)) == 0
 
@@ -124,14 +121,14 @@ async def test_range_phev(caplog, bmw_fixture: respx.Router):
     """Test if the parsing of mileage and range is working."""
     status = (await prepare_account_with_vehicles()).get_vehicle(VIN_G01).fuel_and_battery
 
-    assert (40, "L") == status.remaining_fuel
-    assert (436, "km") == status.remaining_range_fuel
-    assert 80 == status.remaining_fuel_percent
+    assert status.remaining_fuel == (40, "L")
+    assert status.remaining_range_fuel == (436, "km")
+    assert status.remaining_fuel_percent == 80
 
-    assert 80 == status.remaining_battery_percent
-    assert (40, "km") == status.remaining_range_electric
+    assert status.remaining_battery_percent == 80
+    assert status.remaining_range_electric == (40, "km")
 
-    assert (476, "km") == status.remaining_range_total
+    assert status.remaining_range_total == (476, "km")
 
     assert status.remaining_range_fuel[0] + status.remaining_range_electric[0] == status.remaining_range_total[0]
 
@@ -143,14 +140,14 @@ async def test_range_rex(caplog, bmw_fixture: respx.Router):
     """Test if the parsing of mileage and range is working."""
     status = (await prepare_account_with_vehicles()).get_vehicle(VIN_I01_REX).fuel_and_battery
 
-    assert (6, "L") == status.remaining_fuel
-    assert (105, "km") == status.remaining_range_fuel
+    assert status.remaining_fuel == (6, "L")
+    assert status.remaining_range_fuel == (105, "km")
     assert status.remaining_fuel_percent is None
 
-    assert 82 == status.remaining_battery_percent
-    assert (174, "km") == status.remaining_range_electric
+    assert status.remaining_battery_percent == 82
+    assert status.remaining_range_electric == (174, "km")
 
-    assert (279, "km") == status.remaining_range_total
+    assert status.remaining_range_total == (279, "km")
 
     assert status.remaining_range_fuel[0] + status.remaining_range_electric[0] == status.remaining_range_total[0]
 
@@ -166,24 +163,22 @@ async def test_range_electric(caplog, bmw_fixture: respx.Router):
     assert status.remaining_range_fuel == (None, None)
     assert status.remaining_fuel_percent is None
 
-    assert 70 == status.remaining_battery_percent
-    assert (340, "km") == status.remaining_range_electric
+    assert status.remaining_battery_percent == 70
+    assert status.remaining_range_electric == (340, "km")
 
-    assert (340, "km") == status.remaining_range_total
+    assert status.remaining_range_total == (340, "km")
 
     assert len(get_deprecation_warning_count(caplog)) == 0
 
 
-@time_machine.travel("2021-11-28 21:28:59 +0000", tick=False)
+@time_machine.travel("2021-11-28 21:28:59 +0000")
 @pytest.mark.asyncio
 async def test_charging_end_time(caplog, bmw_fixture: respx.Router):
     """Test charging end time."""
     account = await prepare_account_with_vehicles()
     vehicle = account.get_vehicle(VIN_I01_NOREX)
 
-    assert vehicle.fuel_and_battery.charging_end_time.astimezone(UTC) == datetime.datetime(
-        2021, 11, 28, 23, 27, 59, tzinfo=UTC
-    )
+    assert vehicle.fuel_and_battery.charging_end_time == datetime.datetime(2021, 11, 28, 23, 27, 59, tzinfo=UTC)
     assert vehicle.fuel_and_battery.charging_status == ChargingState.CHARGING
     assert vehicle.fuel_and_battery.is_charger_connected is True
     assert vehicle.fuel_and_battery.charging_start_time is None
@@ -191,19 +186,22 @@ async def test_charging_end_time(caplog, bmw_fixture: respx.Router):
     assert len(get_deprecation_warning_count(caplog)) == 0
 
 
-@time_machine.travel("2021-11-28 17:28:59 +0000", tick=False)
+@time_machine.travel("2021-11-28 17:28:59 +0000")
 @pytest.mark.asyncio
 async def test_plugged_in_waiting_for_charge_window(caplog, bmw_fixture: respx.Router):
     """I01_REX is plugged in but not charging, as its waiting for charging window."""
+
+    # Make sure that local timezone for test is UTC
+    os.environ["TZ"] = "Europe/Berlin"
+    time.tzset()
+
     account = await prepare_account_with_vehicles()
     vehicle = account.get_vehicle(VIN_I01_REX)
 
     assert vehicle.fuel_and_battery.charging_end_time is None
     assert vehicle.fuel_and_battery.charging_status == ChargingState.WAITING_FOR_CHARGING
     assert vehicle.fuel_and_battery.is_charger_connected is True
-    assert vehicle.fuel_and_battery.charging_start_time.astimezone(UTC) == datetime.datetime(
-        2021, 11, 28, 18, 1, tzinfo=account.timezone
-    )
+    assert vehicle.fuel_and_battery.charging_start_time == datetime.datetime(2021, 11, 29, 18, 1)
     assert vehicle.fuel_and_battery.charging_target == 100
 
     assert len(get_deprecation_warning_count(caplog)) == 0
@@ -212,24 +210,30 @@ async def test_plugged_in_waiting_for_charge_window(caplog, bmw_fixture: respx.R
 @pytest.mark.asyncio
 async def test_condition_based_services(caplog, bmw_fixture: respx.Router):
     """Test condition based service messages."""
-    vehicle = (await prepare_account_with_vehicles()).get_vehicle(VIN_G26)
+    vehicle = (await prepare_account_with_vehicles()).get_vehicle(VIN_G01)
+
+    assert vehicle.condition_based_services.next_service_by_distance.due_distance == (50000, "km")
+    assert vehicle.condition_based_services.next_service_by_distance.service_type == "BRAKE_FLUID"
+
+    assert vehicle.condition_based_services.next_service_by_time.due_date == datetime.datetime(2024, 11, 1, tzinfo=UTC)
+    assert vehicle.condition_based_services.next_service_by_time.service_type == "VEHICLE_CHECK"
 
     cbs = vehicle.condition_based_services.messages
-    assert 5 == len(cbs)
-    assert ConditionBasedServiceStatus.OK == cbs[0].state
+    assert len(cbs) == 4
+    assert cbs[0].state == ConditionBasedServiceStatus.OK
     expected_cbs0 = datetime.datetime(year=2024, month=12, day=1, tzinfo=UTC)
     assert expected_cbs0 == cbs[0].due_date
-    assert (50000, "km") == cbs[0].due_distance
+    assert cbs[0].due_distance == (50000, "km")
 
-    assert ConditionBasedServiceStatus.OK == cbs[1].state
+    assert cbs[1].state == ConditionBasedServiceStatus.OK
     expected_cbs1 = datetime.datetime(year=2024, month=12, day=1, tzinfo=UTC)
     assert expected_cbs1 == cbs[1].due_date
-    assert (50000, "km") == cbs[1].due_distance
+    assert cbs[1].due_distance == (50000, "km")
 
-    assert ConditionBasedServiceStatus.OK == cbs[2].state
-    expected_cbs2 = datetime.datetime(year=2024, month=12, day=1, tzinfo=UTC)
+    assert cbs[2].state == ConditionBasedServiceStatus.OK
+    expected_cbs2 = datetime.datetime(year=2024, month=11, day=1, tzinfo=UTC)
     assert expected_cbs2 == cbs[2].due_date
-    assert (50000, "km") == cbs[2].due_distance
+    assert cbs[2].due_distance == (50000, "km")
 
     assert vehicle.condition_based_services.is_service_required is False
 
@@ -241,8 +245,8 @@ async def test_position_generic(caplog, bmw_fixture: respx.Router):
     """Test generic attributes."""
     status = (await prepare_account_with_vehicles()).get_vehicle(VIN_G26)
 
-    assert (48.177334, 11.556274) == status.vehicle_location.location
-    assert 180 == status.vehicle_location.heading
+    assert status.vehicle_location.location == (48.177334, 11.556274)
+    assert status.vehicle_location.heading == 180
 
     assert VehicleLocation.from_vehicle_data(status.data).location == status.vehicle_location.location
 
@@ -295,10 +299,10 @@ async def test_parse_gcj02_position(caplog, bmw_fixture: respx.Router):
     # Update twice to test against slowly crawling position due to GCJ02 to WGS84 conversion
     vehicle.update_state(dict(vehicle.data, **vehicle_test_data))
 
-    assert (39.8337, 116.22617) == (
+    assert (
         round(vehicle.vehicle_location.location[0], 5),
         round(vehicle.vehicle_location.location[1], 5),
-    )
+    ) == (39.8337, 116.22617)
 
     assert len(get_deprecation_warning_count(caplog)) == 0
 
@@ -315,16 +319,16 @@ async def test_lids(caplog, bmw_fixture: respx.Router):
     status = (await prepare_account_with_vehicles()).get_vehicle(VIN_G26).doors_and_windows
 
     for lid in status.lids:
-        assert LidState.CLOSED == lid.state
+        assert lid.state == LidState.CLOSED
     assert status.all_lids_closed is True
-    assert 6 == len(list(status.lids))
+    assert len(list(status.lids)) == 6
 
     status = (await prepare_account_with_vehicles()).get_vehicle(VIN_I01_REX).doors_and_windows
 
     for lid in status.lids:
-        assert LidState.CLOSED == lid.state
+        assert lid.state == LidState.CLOSED
     assert status.all_lids_closed is True
-    assert 7 == len(list(status.lids))
+    assert len(list(status.lids)) == 7
 
     assert status.lids[-1].name == "sunRoof"
 
@@ -337,10 +341,10 @@ async def test_windows_g01(caplog, bmw_fixture: respx.Router):
     status = (await prepare_account_with_vehicles()).get_vehicle(VIN_G01).doors_and_windows
 
     for window in status.windows:
-        assert LidState.CLOSED == window.state
+        assert window.state == LidState.CLOSED
 
-    assert 5 == len(list(status.windows))
-    assert 0 == len(list(status.open_windows))
+    assert len(list(status.windows)) == 5
+    assert len(list(status.open_windows)) == 0
     assert status.all_windows_closed is True
 
     assert len(get_deprecation_warning_count(caplog)) == 0
@@ -351,11 +355,11 @@ async def test_door_locks(caplog, bmw_fixture: respx.Router):
     """Test the door locks."""
     status = (await prepare_account_with_vehicles()).get_vehicle(VIN_G01).doors_and_windows
 
-    assert LockState.LOCKED == status.door_lock_state
+    assert status.door_lock_state == LockState.LOCKED
 
     status = (await prepare_account_with_vehicles()).get_vehicle(VIN_I01_REX).doors_and_windows
 
-    assert LockState.UNLOCKED == status.door_lock_state
+    assert status.door_lock_state == LockState.UNLOCKED
 
     assert len(get_deprecation_warning_count(caplog)) == 0
 
@@ -369,22 +373,24 @@ async def test_check_control_messages(caplog, bmw_fixture: respx.Router):
     """
     vehicle = (await prepare_account_with_vehicles()).get_vehicle(VIN_G01)
     assert vehicle.check_control_messages.has_check_control_messages is True
+    assert vehicle.check_control_messages.urgent_check_control_messages == "ENGINE_OIL"
 
     ccms = vehicle.check_control_messages.messages
-    assert 2 == len(ccms)
+    assert len(ccms) == 2
 
-    assert CheckControlStatus.MEDIUM == ccms[1].state
-    assert "ENGINE_OIL" == ccms[1].description_short
+    assert ccms[1].state == CheckControlStatus.MEDIUM
+    assert ccms[1].description_short == "ENGINE_OIL"
     assert None is ccms[1].description_long
 
     vehicle = (await prepare_account_with_vehicles()).get_vehicle(VIN_G20)
     assert vehicle.check_control_messages.has_check_control_messages is False
+    assert vehicle.check_control_messages.urgent_check_control_messages is None
 
     ccms = vehicle.check_control_messages.messages
-    assert 2 == len(ccms)
+    assert len(ccms) == 2
 
-    assert CheckControlStatus.LOW == ccms[1].state
-    assert "ENGINE_OIL" == ccms[1].description_short
+    assert ccms[1].state == CheckControlStatus.LOW
+    assert ccms[1].description_short == "ENGINE_OIL"
     assert None is ccms[1].description_long
 
     assert len(get_deprecation_warning_count(caplog)) == 0
@@ -471,7 +477,7 @@ async def test_tires(bmw_fixture: respx.Router):
     assert tires.rear_left.season == 2
 
 
-@time_machine.travel("2021-11-28 21:28:59 +0000", tick=False)
+@time_machine.travel("2021-11-28 21:28:59 +0000")
 @pytest.mark.asyncio
 async def test_climate(bmw_fixture: respx.Router):
     """Test climate status."""
@@ -492,7 +498,5 @@ async def test_climate(bmw_fixture: respx.Router):
     # Running climatization
     climate = account.get_vehicle(VIN_G26).climate
     assert climate.activity == ClimateActivityState.HEATING
-    assert climate.activity_end_time.astimezone(datetime.timezone.utc) == datetime.datetime(
-        2021, 11, 28, 21, 58, 49, tzinfo=UTC
-    )
+    assert climate.activity_end_time == datetime.datetime(2021, 11, 28, 21, 58, 49, tzinfo=UTC)
     assert climate.is_climate_on is True
